import { Component, Element, Host, Prop, State, Watch, getAssetPath, h } from "@stencil/core";
import { getSvgContent, iconContent } from "./requests";
/*********************************
CONSTANTS
*********************************/
const DEFAULT_COLOR = "onbackground";
const COLOR_MAPPINGS = {
    alwaysblack: "color-always-black",
    disabled: "color-primary-disabled",
    error: "color-error-dark",
    negative: "color-on-primary",
    onbackground: "color-on-background",
    primary: "color-primary-active",
    success: "color-success-dark",
    warning: "color-warning-dark"
};
const CAMEL_CASE_TO_HYPHENED_REGEX = /(.)([A-Z])/g;
export class Icon {
    constructor() {
        /**
         * If enabled, the icon will be loaded lazily when it's visible in the viewport.
         */
        this.lazy = false;
        /**
         * The size of the icon. Possible values: regular, small.
         */
        this.size = "regular";
        this.isVisible = false;
    }
    /*********************************
    METHODS
    *********************************/
    connectedCallback() {
        // purposely do not return the promise here because loading
        // the svg file should not hold up loading the app
        // only load the svg if it's visible
        this.waitUntilVisible(this.element, "50px", () => {
            this.isVisible = true;
            this.getIcon();
        });
    }
    disconnectedCallback() {
        if (this.io) {
            this.io.disconnect();
            this.io = undefined;
        }
    }
    waitUntilVisible(el, rootMargin, callback) {
        if (this.lazy &&
            typeof window !== "undefined" &&
            window.IntersectionObserver) {
            const io = (this.io = new window.IntersectionObserver((data) => {
                if (data[0].isIntersecting) {
                    io.disconnect();
                    this.io = undefined;
                    callback();
                }
            }, { rootMargin }));
            io.observe(el);
        }
        else {
            // browser doesn't support IntersectionObserver
            // so just fallback to always show it
            callback();
        }
    }
    async getIcon() {
        if (this.isVisible) {
            if (this.type === "none") {
                this.svgContent = "";
                return;
            }
            const fileName = this.getFileName();
            const url = getAssetPath(`assets/${fileName}`);
            if (url) {
                if (iconContent.has(url)) {
                    this.svgContent = iconContent.get(url);
                }
                else {
                    this.svgContent = await getSvgContent(url);
                }
            }
        }
    }
    getFileName() {
        const name = this.type
            .replace(CAMEL_CASE_TO_HYPHENED_REGEX, "$1-$2")
            .toLowerCase();
        return `${name}.svg`;
    }
    render() {
        return (h(Host, { "aria-label": this.type },
            h("div", { class: {
                    "svg-icon-native": true
                }, style: {
                    "--icon-color": this.mapColorToCssVar(COLOR_MAPPINGS[this.color]),
                    "--icon-color-default": this.mapColorToCssVar(COLOR_MAPPINGS[DEFAULT_COLOR])
                }, innerHTML: this.svgContent })));
    }
    mapColorToCssVar(color) {
        if (color) {
            return `var(--${color})`;
        }
        return null;
    }
    static get is() { return "gxg-icon"; }
    static get encapsulation() { return "shadow"; }
    static get originalStyleUrls() { return {
        "$": ["icon.scss"]
    }; }
    static get styleUrls() { return {
        "$": ["icon.css"]
    }; }
    static get assetsDirs() { return ["assets"]; }
    static get properties() { return {
        "color": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "Color",
                "resolved": "\"alwaysblack\" | \"auto\" | \"disabled\" | \"error\" | \"negative\" | \"onbackground\" | \"primary\" | \"success\" | \"warning\"",
                "references": {
                    "Color": {
                        "location": "local"
                    }
                }
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "The color of the icon."
            },
            "attribute": "color",
            "reflect": true
        },
        "lazy": {
            "type": "boolean",
            "mutable": false,
            "complexType": {
                "original": "boolean",
                "resolved": "boolean",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "If enabled, the icon will be loaded lazily when it's visible in the viewport."
            },
            "attribute": "lazy",
            "reflect": false,
            "defaultValue": "false"
        },
        "size": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "Size",
                "resolved": "\"regular\" | \"small\"",
                "references": {
                    "Size": {
                        "location": "local"
                    }
                }
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "The size of the icon. Possible values: regular, small."
            },
            "attribute": "size",
            "reflect": true,
            "defaultValue": "\"regular\""
        },
        "type": {
            "type": "any",
            "mutable": false,
            "complexType": {
                "original": "any",
                "resolved": "any",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "The type of icon."
            },
            "attribute": "type",
            "reflect": true
        }
    }; }
    static get states() { return {
        "isVisible": {},
        "svgContent": {}
    }; }
    static get elementRef() { return "element"; }
    static get watchers() { return [{
            "propName": "type",
            "methodName": "getIcon"
        }]; }
}
